==> neptune_widget_toolkit-2.1.0.252.jar: com/tandbergtv/neptune/widgettoolkit/client/widget/composite/table/TableListViewPanel.java <==
/*
 * Created on Sep 14, 2009
 * 
 * (C) Copyright TANDBERG Television Ltd.
 */

package com.tandbergtv.neptune.widgettoolkit.client.widget.composite.table;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.tandbergtv.neptune.widgettoolkit.client.i18n.NeptuneWidgetConstants;
import com.tandbergtv.neptune.widgettoolkit.client.remote.NeptuneAsyncCallback;
import com.tandbergtv.neptune.widgettoolkit.client.widget.basic.LabelWidget;
import com.tandbergtv.neptune.widgettoolkit.client.widget.composite.table.feature.ExpandingFeature;
import com.tandbergtv.neptune.widgettoolkit.client.widget.composite.table.impl.CheckBox;
import com.tandbergtv.neptune.widgettoolkit.client.widget.container.HorizontalContainer;
import com.tandbergtv.neptune.widgettoolkit.client.widget.container.SimpleContainer;
import com.tandbergtv.neptune.widgettoolkit.client.widget.container.VerticalContainer;

/**
 * The table list view panel
 * 
 * @author Vijay Silva
 */
class TableListViewPanel<K, R extends Record<K>> extends Composite {

	/* Properties */
	private final DataProvider<K, R> dataProvider;
	private TableColumnPanel<K, R> columnPanel;
	private HorizontalContainer recordCountPanel;
	private LabelWidget numRecordsLabel;
	private HorizontalContainer actionContainer;
	private SimpleContainer paginationPanel;
	private List<Listener> listeners;
	private VerticalContainer mainContainer;
	private ExpandingFeature<K,R> expandingFeature;
	private Integer totalRecordCount = null;
	protected NeptuneWidgetConstants constants = GWT.create(NeptuneWidgetConstants.class);
	
	/**
	 * Constructor
	 * 
	 * @param dataProvider The data provider
	 */
	public TableListViewPanel(DataProvider<K, R> dataProvider) {
		this(dataProvider, null);
	}
	
	/**
	 * Constructor
	 * 
	 * @param dataProvider The data provider
	 * @param expandingFeature The expanding feature
	 */
	public TableListViewPanel(DataProvider<K, R> dataProvider, ExpandingFeature<K,R> expandingFeature) {
		this.dataProvider = dataProvider;
		this.expandingFeature = expandingFeature;
		initializeWidget();
	}

	// ========================================================================
	// ===================== INITIALIZATION
	// ========================================================================

	/*
	 * Initialize the widget
	 */
	private void initializeWidget() {
		listeners = new ArrayList<Listener>();
		mainContainer = new VerticalContainer();

		/* create place holder panels even if the feature is not enabled */
		recordCountPanel = new HorizontalContainer();
		numRecordsLabel = new LabelWidget("");
		Label numRecordsLabelFront = new Label(constants.recordsLabel());
		recordCountPanel.add(numRecordsLabelFront);
		recordCountPanel.add(numRecordsLabel);
		actionContainer = new HorizontalContainer();
		paginationPanel = new SimpleContainer();
		columnPanel = new TableColumnPanel<K, R>(dataProvider, expandingFeature);

		/* set style information for the list view and its sections */ 
		mainContainer.setStylePrimaryName(TableConstants.STYLE_TABLE_PANEL);
		mainContainer.setSpacing(TableConstants.PANEL_SPACING);
		recordCountPanel.setStylePrimaryName(TableConstants.STYLE_TABLE_RECORDCOUNT_PANEL);
		recordCountPanel.setVisible(dataProvider.isRecordCountEnabled());
		//numRecordsLabel.setStyleName(TableConstants.STYLE_TABLE_RECORDCOUNT_LABEL);
		//numRecordsLabel.addStyleName(TableConstants.STYLE_GENERAL_TEXT);
		actionContainer.setStylePrimaryName(TableConstants.STYLE_TABLE_ACTION_PANEL);
		paginationPanel.setStylePrimaryName(TableConstants.STYLE_TABLE_PAGING_PANEL);
		
//		numRecordsLabelFront.setStyleName("ebLayout-SectionLabel");
//		numRecordsLabel.setStyleName("ebText_alternative");
//		
		numRecordsLabelFront.setStyleName("nwt_HeaderSection_label");
     	numRecordsLabel.setStyleName("nwt_HeaderSection_count");

		/* add widgets to the panel */
		mainContainer.add(recordCountPanel);
		mainContainer.add(columnPanel);
		mainContainer.add(actionContainer);
		mainContainer.add(paginationPanel);

		initWidget(mainContainer);
	}

	// ========================================================================
	// ===================== API
	// ========================================================================

	public HorizontalPanel getHeaderColPanelForColumn(Column<?, R> column) {
		return columnPanel.getHeaderColumnPanel(column);
	}

	public Label getHeaderLabelForColumn(Column<?, R> column) {
		return columnPanel.getHeaderColumnLabel(column);
	}

	/**
	 * Refresh will cause an asynchronous update of the current data
	 */
	public void refresh() {
		/* Clear the previous records, and request for the new records */
		dataProvider.getRecords(new NeptuneAsyncCallback<List<R>>() {
			public void onNeptuneFailure(Throwable caught) {
				Window.alert(caught.getLocalizedMessage());

				/* Clear all the records, notify listeners */
				resetTotalRecordCount();
				columnPanel.updateRecords(new ArrayList<R>());
				notifyAllListeners();
			}

			public void onNeptuneSuccess(List<R> result) {
				/* Display all the records and notify listeners */
				setTotalRecordCount((result != null) ? result.size() : 0);
				columnPanel.updateRecords(result);
				notifyAllListeners();
			}
		});
	}

	public void clearRecords() {
		resetTotalRecordCount();
		columnPanel.clearRecords();
	}

	public Integer getTotalRecordCount() {
	    return this.totalRecordCount;
    }

	public void setTotalRecordCount(Integer recordCount) {
	    if (recordCount == null || recordCount < 0) {
	    	resetTotalRecordCount();
	    } else {
	    	this.totalRecordCount = recordCount;
	    	updateRecordCountLabel();
	    }
    }

	private void resetTotalRecordCount() {
		this.totalRecordCount = null;
		updateRecordCountLabel();
	}

	private void updateRecordCountLabel() {
		if (this.totalRecordCount == null)
			this.numRecordsLabel.setText("");
		else
			this.numRecordsLabel.setText("("+this.totalRecordCount+")");
	}

	public SimpleContainer getPaginationPanel() {
		return paginationPanel;
	}

    public HorizontalContainer getRecordCountPanel() {
        return recordCountPanel;
    }

    public List<Column<?, R>> listVisibleColumns() {
		return columnPanel.listVisibleColumns();
	}

	public List<Widget> listWidgetsForVisibleColumn(Column<?, R> visibleColumn) {
		return columnPanel.listWidgetsForVisibleColumn(visibleColumn);
	}

	public R getRecordForWidget(Widget widget) {
		return columnPanel.getRecordForWidget(widget);
	}
	
	/**
	 * Get the data rows checkboxes
	 * 
	 * @return The data rows checkboxes
	 */
	public List<CheckBox<R>> getDataRowsCheckBoxes() {
		return columnPanel.getDataRowsCheckBoxes();
	}

	/**
	 * Get the header row check box
	 * 
	 * @return The header row check box
	 */
	public CheckBox<R> getHeaderRowCheckBox() {
		return columnPanel.getHeaderRowCheckBox();
	}

	public HorizontalContainer getActionContainer() {
		return this.actionContainer;
	}
	
	public List<R> getRecords() {
		return columnPanel.getRecords();
	}

	public List<R> getSelectedRecords() {
		return columnPanel.listSelectedRecords();
	}
	
	public int getDataRowCount() {
		return columnPanel.getDataRowCount();
	}

	public void addListener(Listener listener) {
		if (listener != null && !listeners.contains(listener))
			listeners.add(listener);
	}

	public void removeListener(Listener listener) {
		listeners.remove(listener);
	}

	private void notifyAllListeners() {
		for (Listener listener : listeners)
			listener.updated();
	}

	public void setExpandingFeature(ExpandingFeature<K,R> expandingFeature) {
		this.expandingFeature = expandingFeature;
	}

	public ExpandingFeature<K,R> getExpandingFeature() {
		return expandingFeature;
	}
	
	/**
	 * Add a listener for selection change events triggered by the table
	 * 
	 * @param handler the event handler
	 */
	public void addSelectionChangeHandler(SelectionChangeHandler<K,R> handler) {
		columnPanel.addSelectionChangeHandler( handler );
	}

	/**
	 * Remove a listener for selection change events triggered by the table
	 * 
	 * @param handler the event handler
	 */
	public void removeSelectionChangeHandler(SelectionChangeHandler<K,R> handler) {
		columnPanel.removeSelectionChangeHandler( handler );
	}	
	
	public TableColumnPanel<K,R> getTableColumnPanel() {
		return this.columnPanel;
	}

	public interface Listener {
		void updated();
	}
}

==> neptune_widget_toolkit-5.2.0.186.jar: com/tandbergtv/neptune/widgettoolkit/client/widget/composite/table/TableListViewPanel.java <==
/*
 * Created on Sep 14, 2009
 * 
 * (C) Copyright TANDBERG Television Ltd.
 */

package com.tandbergtv.neptune.widgettoolkit.client.widget.composite.table;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.tandbergtv.neptune.widgettoolkit.client.i18n.NeptuneWidgetConstants;
import com.tandbergtv.neptune.widgettoolkit.client.remote.NeptuneAsyncCallback;
import com.tandbergtv.neptune.widgettoolkit.client.widget.basic.LabelWidget;
import com.tandbergtv.neptune.widgettoolkit.client.widget.composite.table.feature.ExpandingFeature;
import com.tandbergtv.neptune.widgettoolkit.client.widget.composite.table.impl.CheckBox;
import com.tandbergtv.neptune.widgettoolkit.client.widget.container.HorizontalContainer;
import com.tandbergtv.neptune.widgettoolkit.client.widget.container.SimpleContainer;
import com.tandbergtv.neptune.widgettoolkit.client.widget.container.VerticalContainer;

/**
 * The table list view panel
 * 
 * @author Vijay Silva
 */
class TableListViewPanel<K, R extends Record<K>> extends Composite {

	/* Properties */
	private final DataProvider<K, R> dataProvider;
	private TableColumnPanel<K, R> columnPanel;
    private HorizontalContainer topPanel;
    private HorizontalContainer bottomPanel;
	private LabelWidget numRecordsLabel;
	private HorizontalContainer actionContainer;
	private SimpleContainer paginationPanel;
	private List<Listener> listeners;
	private VerticalContainer mainContainer;
	private ExpandingFeature<K,R> expandingFeature;
	private Integer totalRecordCount = null;
	protected NeptuneWidgetConstants constants = GWT.create(NeptuneWidgetConstants.class);
	
	/**
	 * Constructor
	 * 
	 * @param dataProvider The data provider
	 */
	public TableListViewPanel(DataProvider<K, R> dataProvider) {
		this(dataProvider, null);
	}
	
	/**
	 * Constructor
	 * 
	 * @param dataProvider The data provider
	 * @param expandingFeature The expanding feature
	 */
	public TableListViewPanel(DataProvider<K, R> dataProvider, ExpandingFeature<K,R> expandingFeature) {
		this.dataProvider = dataProvider;
		this.expandingFeature = expandingFeature;
		initializeWidget();
	}

	// ========================================================================
	// ===================== INITIALIZATION
	// ========================================================================

	/*
	 * Initialize the widget
	 */
	private void initializeWidget() {
		listeners = new ArrayList<Listener>();
		mainContainer = new VerticalContainer();

        initTopPanel();
        initBottomPanel();

		actionContainer = new HorizontalContainer();
		columnPanel = new TableColumnPanel<K, R>(dataProvider, expandingFeature);

		/* set style information for the list view and its sections */ 
		mainContainer.setStylePrimaryName(TableConstants.STYLE_TABLE_PANEL);
		mainContainer.setSpacing(TableConstants.PANEL_SPACING);
		//numRecordsLabel.setStyleName(TableConstants.STYLE_TABLE_RECORDCOUNT_LABEL);
		//numRecordsLabel.addStyleName(TableConstants.STYLE_GENERAL_TEXT);
		actionContainer.setStylePrimaryName(TableConstants.STYLE_TABLE_ACTION_PANEL);
		
//		numRecordsLabelFront.setStyleName("ebLayout-SectionLabel");
//		numRecordsLabel.setStyleName("ebText_alternative");
//		

		/* add widgets to the panel */
        mainContainer.add(topPanel);
		mainContainer.add(columnPanel);
		mainContainer.add(actionContainer);
        mainContainer.add(bottomPanel);

		initWidget(mainContainer);
	}

    private void initTopPanel() {
        /* create place holder panels even if the feature is not enabled */
        numRecordsLabel = new LabelWidget("");
        numRecordsLabel.setStyleName("nwt_HeaderSection_count");
        Label numRecordsLabelFront = new Label(constants.recordsLabel());
        numRecordsLabelFront.setStyleName("nwt_HeaderSection_label");

        HorizontalContainer recordCountContainer = new HorizontalContainer();
        recordCountContainer.add(numRecordsLabelFront);
        recordCountContainer.add(numRecordsLabel);

        topPanel = new HorizontalContainer();
        topPanel.add(recordCountContainer);
        topPanel.setStylePrimaryName(TableConstants.STYLE_TABLE_RECORDCOUNT_PANEL);
        topPanel.setVisible(dataProvider.isRecordCountEnabled());
        topPanel.setWidth("100%");
    }

    private void initBottomPanel() {
        paginationPanel = new SimpleContainer();
        paginationPanel.setStylePrimaryName(TableConstants.STYLE_TABLE_PAGING_PANEL);

        bottomPanel = new HorizontalContainer();
        bottomPanel.add(paginationPanel);
        bottomPanel.setWidth("100%");
    }

	// ========================================================================
	// ===================== API
	// ========================================================================

	public HorizontalPanel getHeaderColPanelForColumn(Column<?, R> column) {
		return columnPanel.getHeaderColumnPanel(column);
	}

	public Label getHeaderLabelForColumn(Column<?, R> column) {
		return columnPanel.getHeaderColumnLabel(column);
	}

	/**
	 * Refresh will cause an asynchronous update of the current data
	 */
	public void refresh() {
		/* Clear the previous records, and request for the new records */
		dataProvider.getRecords(new NeptuneAsyncCallback<List<R>>() {
			public void onNeptuneFailure(Throwable caught) {
				Window.alert(caught.getLocalizedMessage());

				/* Clear all the records, notify listeners */
				resetTotalRecordCount();
				columnPanel.updateRecords(new ArrayList<R>());
				notifyAllListeners();
			}

			public void onNeptuneSuccess(List<R> result) {
				/* Display all the records and notify listeners */
				setTotalRecordCount((result != null) ? result.size() : 0);
				columnPanel.updateRecords(result);
				notifyAllListeners();
			}
		});
	}

	public void clearRecords() {
		resetTotalRecordCount();
		columnPanel.clearRecords();
	}

	public Integer getTotalRecordCount() {
	    return this.totalRecordCount;
    }

	public void setTotalRecordCount(Integer recordCount) {
	    if (recordCount == null || recordCount < 0) {
	    	resetTotalRecordCount();
	    } else {
	    	this.totalRecordCount = recordCount;
	    	updateRecordCountLabel();
	    }
    }

	private void resetTotalRecordCount() {
		this.totalRecordCount = null;
		updateRecordCountLabel();
	}

	private void updateRecordCountLabel() {
		if (this.totalRecordCount == null)
			this.numRecordsLabel.setText("");
		else
			this.numRecordsLabel.setText("("+this.totalRecordCount+")");
	}

	public SimpleContainer getPaginationPanel() {
		return paginationPanel;
	}

    public List<Column<?, R>> listVisibleColumns() {
		return columnPanel.listVisibleColumns();
	}

	public List<Widget> listWidgetsForVisibleColumn(Column<?, R> visibleColumn) {
		return columnPanel.listWidgetsForVisibleColumn(visibleColumn);
	}

	public R getRecordForWidget(Widget widget) {
		return columnPanel.getRecordForWidget(widget);
	}
	
	/**
	 * Get the data rows checkboxes
	 * 
	 * @return The data rows checkboxes
	 */
	public List<CheckBox<R>> getDataRowsCheckBoxes() {
		return columnPanel.getDataRowsCheckBoxes();
	}

	/**
	 * Get the header row check box
	 * 
	 * @return The header row check box
	 */
	public CheckBox<R> getHeaderRowCheckBox() {
		return columnPanel.getHeaderRowCheckBox();
	}

	public HorizontalContainer getActionContainer() {
		return this.actionContainer;
	}
	
	public List<R> getRecords() {
		return columnPanel.getRecords();
	}

	public List<R> getSelectedRecords() {
		return columnPanel.listSelectedRecords();
	}
	
	public int getDataRowCount() {
		return columnPanel.getDataRowCount();
	}

	public void addListener(Listener listener) {
		if (listener != null && !listeners.contains(listener))
			listeners.add(listener);
	}

	public void removeListener(Listener listener) {
		listeners.remove(listener);
	}

	private void notifyAllListeners() {
		for (Listener listener : listeners)
			listener.updated();
	}

	public void setExpandingFeature(ExpandingFeature<K,R> expandingFeature) {
		this.expandingFeature = expandingFeature;
	}

	public ExpandingFeature<K,R> getExpandingFeature() {
		return expandingFeature;
	}
	
	/**
	 * Add a listener for selection change events triggered by the table
	 * 
	 * @param handler the event handler
	 */
	public void addSelectionChangeHandler(SelectionChangeHandler<K,R> handler) {
		columnPanel.addSelectionChangeHandler( handler );
	}

	/**
	 * Remove a listener for selection change events triggered by the table
	 * 
	 * @param handler the event handler
	 */
	public void removeSelectionChangeHandler(SelectionChangeHandler<K,R> handler) {
		columnPanel.removeSelectionChangeHandler( handler );
	}	
	
	public TableColumnPanel<K,R> getTableColumnPanel() {
		return this.columnPanel;
	}

	public interface Listener {
		void updated();
	}

    public HorizontalContainer getTopPanel() {
        return topPanel;
    }

    public HorizontalContainer getBottomPanel() {
        return bottomPanel;
    }

}
